# -*- coding: utf-8 -*-
""" FHCoreLibs > ADMIN > Base.py by Frankhood
    @author: Frankhood Business Solutions
"""
import datetime
import os
import posixpath

from django.db.models import Q
from django.conf import settings
from django.contrib import admin
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.utils.translation import ungettext, ugettext_lazy as _

from ..forms import AdvancedEntryModelForm, BaseEntryModelForm
from ..models.mixins import DRAFT, PUBLISHED
from ..admin.mixins import OrderableModelAdmin, PublicableModelAdmin, LocaleModelAdmin
from fhcore.admin.mixins import ConfigurePreviewMixinAdmin
from fhcore.forms import AdvancedLocaleModelForm


class BaseModelAdmin(admin.ModelAdmin):
    """
    A simple base model admin with a predefined behaviour 
    related Model: core.models.base.StdBaseModel
    
    """
    list_per_page = 25
    save_as = True
    date_hierarchy = 'created'
    
    def save_model(self, request, obj, form, change):
        if not change:
            obj.created_by = request.user and request.user.username
        obj.last_modified_by = request.user and request.user.username
        super(BaseModelAdmin, self).save_model(request, obj, form, change)


class BaseCategoryModelAdmin(BaseModelAdmin):
    search_fields = ['name']
    prepopulated_fields = {'slug': ("name",)}
    
class BaseOrderedCategoryModelAdmin(OrderableModelAdmin, BaseCategoryModelAdmin):
    order_field = 'sort_num'
    ordering = ('sort_num',)

class AdvancedCategoryModelAdmin(PublicableModelAdmin, BaseOrderedCategoryModelAdmin):
    pass

class BaseEntryModelAdmin(BaseModelAdmin):
    search_fields = ['title']
    prepopulated_fields = {'slug': ("title",)}
    form = BaseEntryModelForm

class PublicableEntryModelAdmin(PublicableModelAdmin, BaseEntryModelAdmin):
    form = BaseEntryModelForm

class AdvancedLocaleModelAdmin(OrderableModelAdmin, LocaleModelAdmin, PublicableEntryModelAdmin):
    filter_horizontal = ['sites']
    form = AdvancedLocaleModelForm
    
class AdvancedEntryModelAdmin(PublicableModelAdmin, BaseEntryModelAdmin): #OrderableModelAdmin, 
    """ To be used as an abstract class """
    form = AdvancedEntryModelForm
    save_on_top= False
    date_hierarchy = 'publication_start'
    
    def get_tags_as_string(self, obj):
        tag_list = obj.tag_list
        return ', '.join(tag_list) if len(tag_list) else '--'
    get_tags_as_string.short_description = _("tags")
    
    def get_sites_as_string(self, obj):
        return ', '.join(obj.sites.values_list('name', flat=True))
    get_sites_as_string.short_description = _("published on")
    
    def make_published(self, request, queryset):
        current_site = Site.objects.filter(id=settings.SITE_ID)
        rows_updated = queryset.update(status=PUBLISHED, publication_start=datetime.datetime.now())
        for item in queryset:
            if not item.sites.count():
                item.sites.add(current_site)
        self.message_user(request, ungettext('%(count)d entries was published', 
                                             '%(count)d entries where published', 
                                             rows_updated) % {'count': rows_updated})
    make_published.short_description = _('Publish selected entries')

    def make_unpublished(self, request, queryset):
        rows_updated = queryset.update(status=DRAFT, publication_start=None)
        self.message_user(request, ungettext('%(count)d entries was unpublished', 
                                             '%(count)d entries where unpublished', 
                                             rows_updated) % {'count': rows_updated})
    make_unpublished.short_description = _('Unpublish selected entries')
    
    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        """ preselect the current logged user as the author of the entry """
        if db_field.name == 'author':
            user = request.user
            app_label = self.model._meta.app_label
            qs_filter_clause = [ Q(is_superuser=True) | Q(is_staff=True) ] #TODO: must be completed
            #kwargs['queryset'] = Author.objects.filter(*qs_filter_clause)
            kwargs['queryset'] = User.objects.all()
            if user.has_module_perms(app_label):
                kwargs['initial'] = request.user
        return super(AdvancedEntryModelAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        """ preselect the current site """
        if db_field.name == 'sites':
            current_site = Site.objects.filter(id=settings.SITE_ID) #should return only one element
            kwargs['initial'] = current_site
        return super(AdvancedEntryModelAdmin, self).formfield_for_manytomany(db_field, request, **kwargs)


    
class AdvancedItemModelAdmin(LocaleModelAdmin, OrderableModelAdmin, AdvancedEntryModelAdmin):
    ''''''

class AdvancedArticleModelAdmin(LocaleModelAdmin, AdvancedEntryModelAdmin):
    ''''''
