# -*- coding: utf-8 -*-
""" FHCoreLibs > ADMIN > Base.py by Frankhood
    @author: Frankhood Business Solutions
"""
import datetime
import os

from django.db.models import Q
from django.conf import settings
from django.conf.urls import *
from django.contrib import admin, messages
from django.contrib.admin.filters import SimpleListFilter
from django.contrib.admin.views.main import ChangeList
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned
from django.http import HttpResponseRedirect, HttpResponse, Http404
from django.utils.encoding import force_unicode
from django.utils.html import escape, escapejs
from django.utils.translation import ungettext, ugettext as _

from ..forms import AdvancedEntryModelForm
from ..models.mixins import DRAFT, PUBLISHED

if settings.USE_TZ:
    from django.utils.timezone import now
else:
    from datetime.datetime import now

class isPublished(SimpleListFilter):
    title = _('Published')
    parameter_name = 'is_published'
    
    def lookups(self, request, model_admin):
        """
        Returns a list of tuples. The first element in each
        tuple is the coded value for the option that will
        appear in the URL query. The second element is the
        human-readable name for the option that will appear
        in the right sidebar.
        """
        return (
            ('True', _('Yes')),
            ('False', _('No')),
        )

    def queryset(self, request, queryset):
        """
        Returns the filtered queryset based on the value
        provided in the query string and retrievable via
        `self.value()`.
        """
        if self.value() == 'True' or self.value() == 'False':
            ids = []
            for obj in queryset:
                if obj.is_actual:
                    ids.append(obj.id) 
            if self.value() == 'True':
                return queryset.filter(id__in = ids)
            else:
                return queryset.exclude(id__in = ids)
        if self.value():
            messages.add_message(request, messages.WARNING, _("Impossible to filter with the argument '{0}' for the key 'Published'").format(self.value()))


class PublicableModelAdmin(admin.ModelAdmin):
    publication_field = 'status'
    date_hierarchy = 'publication_start'
    actions = ['make_published', 'make_unpublished']
    
    def __init__(self, *args, **kwargs):
        super(PublicableModelAdmin, self).__init__(*args, **kwargs)
        if self.get_actual_status not in self.list_display:
            self.list_display = list(self.list_display) + [self.get_actual_status]
        #if self.get_actual_status not in self.list_editable:
        #    self.list_editable = list(self.list_editable) + [self.publication_field]
        if isPublished not in self.list_filter:
            self.list_filter = list(self.list_filter) + [isPublished]
        
    def get_actual_status(self, obj):
        return obj.is_actual
    get_actual_status.short_description = _('is published')
    get_actual_status.boolean = True
    
    
    def make_published(self, request, queryset):
        rows_updated = queryset.update(status=PUBLISHED, publication_start=now(), publication_end=None)
        self.message_user(request, ungettext('%(count)d entries was published', 
                                             '%(count)d entries where published', 
                                             rows_updated) % {'count': rows_updated})
    make_published.short_description = _('Publish selected entries')

    def make_unpublished(self, request, queryset):
        rows_updated = queryset.update(status=DRAFT, publication_start=None)
        self.message_user(request, ungettext('%(count)d entries was unpublished', 
                                             '%(count)d entries where unpublished', 
                                             rows_updated) % {'count': rows_updated})
    make_unpublished.short_description = _('Unpublish selected entries')
   
    
    
class OrderableModelAdmin(admin.ModelAdmin):
    order_field = 'order'
    ordering = ('order',)
    
    def __init__(self, *args, **kwargs):
        super(OrderableModelAdmin, self).__init__(*args, **kwargs)
        if self.order_field not in self.list_display:
            self.list_display = list(self.list_display) + [self.order_field]
        if self.order_field not in self.list_editable:
            self.list_editable = list(self.list_editable) + [self.order_field]

    def _media(self):
        js = (os.path.join(settings.STATIC_URL, 'fhcore/common/js/jquery-1.7.2.min.js'), 
              os.path.join(settings.STATIC_URL, 'fhcore/common/js/jquery-ui-1.8.20.custom.min.js'),
              os.path.join(settings.STATIC_URL, 'fhcore/admin/js/orderable.js'))
        css= {'all': (os.path.join(settings.STATIC_URL,'fhcore/common/css/ui-lightness/jquery-ui-1.8.20.custom.css'),)}
        currmedia = super(OrderableModelAdmin, self).media
        #currmedia = super(OrderableModelAdmin, self)._media() # active in django 1.3.x
        old_js=list(currmedia._js)
        old_js.extend(js)
        currmedia._js=tuple(old_js)
        old_css=list(currmedia._css.get('all',[]))
        old_css.extend(css['all'])
        currmedia._css['all']=old_css
        return currmedia
    media = property(_media)
    
class LocaleModelAdmin(admin.ModelAdmin):
    locale_field = 'lang'
    
    def __init__(self, *args, **kwargs):
        super(LocaleModelAdmin, self).__init__(*args, **kwargs)
        if self.locale_field not in self.list_display:
            self.list_display = list(self.list_display) + [self.locale_field]
        if self.locale_field not in self.list_filter:
            self.list_filter = list(self.list_filter) + [self.locale_field]
        if self.locale_field not in self.search_fields:
            self.search_fields = list(self.search_fields) + [self.locale_field]

    def save_form(self, request, form, change):
        """
        Given a ModelForm return an unsaved instance. ``change`` is True if
        the object is being changed, and False if it's being added.
        """
        if request.method == 'POST' and "_saveasnew" in request.POST:
            if form.cleaned_data.has_key('status') and self.locale_field in form.changed_data:
                form.cleaned_data['status']= DRAFT
        return super(LocaleModelAdmin,self).save_form(request, form, change)

class CountModelAdmin(admin.ModelAdmin):
    order_field = 'counter'
    ordering = ('-counter',)
    
    def __init__(self, *args, **kwargs):
        super(CountModelAdmin, self).__init__(*args, **kwargs)
        if self.order_field not in self.list_display:
            self.list_display = list(self.list_display) + [self.order_field]

class NoPreviewChangeList(ChangeList):
    preview_variable = 'is_preview'
    
    def get_query_set(self):
        qs = super(NoPreviewChangeList,self).get_query_set()
        if hasattr(self,'preview_variable'):
            qs = qs.filter(**({getattr(self,'preview_variable'):False}))
        return qs
        
        
class ConfigureRedirectMixinAdmin(object):
    
    def get_changelist(self, request, **kwargs):
        """
        Returns the ChangeList class for use on the changelist page.
        """
        return NoPreviewChangeList
    
    
    def redirect_for_continue(self, request, obj, post_url, msg=None, add=False):
        if msg:
            self.message_user(request, msg + ' ' + _("You may edit it again below."))        
        return HttpResponseRedirect(post_url)
    
    def redirect_for_popup(self, request, obj, post_url=None, msg=None, add=False):       
        pk_value = obj._get_pk_val()        
        if post_url:
            return HttpResponseRedirect(post_url)
        else:
            return HttpResponse('<script type="text/javascript">opener.dismissAddAnotherPopup(window, "%s", "%s");</script>' % \
                    # escape() calls force_unicode.
                    (escape(pk_value), escapejs(obj)))
        
    def redirect_for_saveasnew(self, request, obj, post_url, msg=None, add=False):
        return HttpResponseRedirect(post_url)
    
    def redirect_for_saveanother(self, request, obj, post_url, msg=None, add=False):
        return HttpResponseRedirect(post_url)
    
    def redirect_for_addanother(self, request, obj, post_url="../add/", msg=None, add=False):
        return HttpResponseRedirect(post_url)
    
    def redirect_for_other(self, request, obj, msg, add):
        self.message_user(request, msg)

        # Figure out where to redirect. If the user has change permission,
        # redirect to the change-list page for this object. Otherwise,
        # redirect to the admin index.
        if self.has_change_permission(request, None):
            post_url = '../'
        else:
            post_url = '../../../'
        return HttpResponseRedirect(post_url)
    
    def response_add(self, request, obj, post_url_continue='../%s/'):
        """
        Determines the HttpResponse for the add_view stage.
        """
        opts = obj._meta
        pk_value = obj._get_pk_val()

        msg = _('The %(name)s "%(obj)s" was added successfully.') % {'name': force_unicode(opts.verbose_name), 'obj': force_unicode(obj)}
        # Here, we distinguish between different save types by checking for
        # the presence of keys in request.POST.
        if request.POST.has_key("_continue"):
            if request.POST.has_key("_popup"):
                post_url_continue += "?_popup=1"    
            return self.redirect_for_continue(request=request, obj=obj, post_url=post_url_continue % pk_value, msg=msg, add=True)
        if request.POST.has_key("_popup"):
            return self.redirect_for_popup(request=request, obj=obj, msg=msg, add=True)
        elif request.POST.has_key("_addanother"):
            self.message_user(request, msg + ' ' + (_("You may add another %s below.") % force_unicode(opts.verbose_name)))
            return self.redirect_for_saveanother(request=request, obj=obj, post_url=request.path, msg=msg, add=True)
        else:
            return self.redirect_for_other(request=request, obj=obj, msg=msg, add=True)
            

    def response_change(self, request, obj):
        """
        Determines the HttpResponse for the change_view stage.
        """
        opts = obj._meta
        pk_value = obj._get_pk_val()

        msg = _('The %(name)s "%(obj)s" was changed successfully.') % {'name': force_unicode(opts.verbose_name), 'obj': force_unicode(obj)}
        if request.POST.has_key("_continue"):
            if request.REQUEST.has_key('_popup'):
                return self.redirect_for_continue(request=request, obj=obj, post_url=request.path + "?_popup=1", msg=msg, add=False)
            else:
                return self.redirect_for_continue(request=request, obj=obj, post_url=request.path, msg=msg, add=False)
        elif request.POST.has_key("_saveasnew"):
            msg = _('The %(name)s "%(obj)s" was added successfully. You may edit it again below.') % {'name': force_unicode(opts.verbose_name), 'obj': obj}
            self.message_user(request, msg)
            return self.redirect_for_saveasnew(request=request, obj=obj, post_url="../%s/" % pk_value, msg=msg, add=False) 
        elif request.POST.has_key("_addanother"):
            self.message_user(request, msg + ' ' + (_("You may add another %s below.") % force_unicode(opts.verbose_name)))
            return self.redirect_for_addanother(request=request, obj=obj, msg=msg, add=False)        
        else:
            return self.redirect_for_other(request=request, obj=obj, msg=msg, add=False)

class ConfigurePreviewMixinAdmin(ConfigureRedirectMixinAdmin):
    
    save_add = False
    save_as = False
    preview_variable = 'is_preview'
    preview_view = None #ogni admin che lo eredita deve inserire la vista di preview da richiamare dal js
    preview_reverse_name = None #ogni admin che lo eredita deve inserire il reverse url per scopi futuri 
    inline_fk_name = "related_entries"#None
    inline_fk_reverse = "photo_gallery"#None
    
    class Media:
        js = (os.path.join(settings.STATIC_URL, 'fhcore/common/js/jquery-1.6.2.min.js'),
              os.path.join(settings.STATIC_URL, 'fhcore/admin/js/is_preview.js'))
    
    def redirect_for_continue(self, request, obj, post_url, msg=None, add=False):
        if msg:
            self.message_user(request, msg + ' ' + _("You may edit it again below."))
        new_url = '/'.join(post_url.split('/')[:-2])+'/%s/' % obj.id        
        return HttpResponseRedirect(new_url)
    
    
    def get_urls(self):
        urls = super(ConfigurePreviewMixinAdmin, self).get_urls()
        my_urls = patterns('',
                           url(r'^preview/(?P<pk>\d+)/$', self.admin_site.admin_view(self.preview_view.as_view(has_preview_permission=True)), name=self.preview_reverse_name)
                           )
        return my_urls + urls
    
    def queryset(self, request):
        qs = super(ConfigurePreviewMixinAdmin,self).queryset(request)
        return qs
    
    def get_old_obj_kw(self, obj, **kwargs):
        bool = kwargs.get(self.preview_variable,None)
        tuples = self.model._meta.unique_together
        filter_kw = {}
        for tupla in tuples:
            if (bool is not None) and self.preview_variable in tupla:
                filter_kw.update({self.preview_variable:bool})
            for item in tupla:
                if self.preview_variable <> item:
                    filter_kw.update({item:getattr(obj,item)}) 
            break
        return filter_kw

    
    def save_model(self, request, obj, form, change):
        self.preview_enter = False
        self.old_obj = None
        self.curr_form = form
        try:
            kw = {self.preview_variable : False}
            old_obj = self.model._default_manager.get(**(self.get_old_obj_kw(obj,**kw)))
            #old_obj = self.model.objects.get(is_preview=False, slug = obj.slug, lang = obj.lang)
        except ObjectDoesNotExist:
            old_obj = None
        except MultipleObjectsReturned:
            raise Http404(u'You have to specify unique together in your model with preview variable into it')
        try:
            kw = {self.preview_variable : True}
            old_preview_obj = self.model._default_manager.get(**(self.get_old_obj_kw(obj,**kw)))
        except ObjectDoesNotExist:
            old_preview_obj = None
        except MultipleObjectsReturned:
            raise Http404(u'You have to specify unique together in your model with preview variable into it')
        if request.POST.has_key('_is_preview'):
            if old_obj:
                if getattr(obj,self.preview_variable)==False:
                    # Sto attivando la preview
                    obj.id = None
                    self.preview_enter = True
            if old_preview_obj and (old_preview_obj.id <> obj.id) :
                old_preview_obj.delete()
            setattr(obj,self.preview_variable,True)
        else:
            if getattr(obj,self.preview_variable)==True :
                if old_obj:
                    self.old_obj = old_obj
            #setattr(obj,self.preview_variable,False)
        obj.save()
        #super(ConfigurePreviewMixinAdmin,self).save_model(request, obj, form, change)
    
    def save_formset(self, request, form, formset, change):
        """
        Given an inline formset save it to the database.
        """
        obj = form.instance
        if request.POST.has_key('_is_preview') or getattr(obj,self.preview_variable):
            frms = formset.save(commit=False)
            for f in formset.forms:
                if f and f.instance and getattr(f.instance, self.inline_fk_name, None):
                    inst = f.instance
                    setattr(inst, self.inline_fk_name, obj) 
                    if self.preview_enter:
                        inst.id = None
                    inst.save()
            for inst in frms:
                setattr(inst, self.inline_fk_name, obj) 
                inst.save()
        elif self.old_obj:
            # Merge tra il vecchio oggetto e il nuovo
            frms = formset.save(commit=False)
            for old_inline in getattr(self.old_obj, self.inline_fk_reverse).all(): 
                old_inline.delete()
            for f in formset.forms:
                if f and f.instance and getattr(f.instance, self.inline_fk_name, None):
                    inst = f.instance 
                    setattr(inst, self.inline_fk_name, self.old_obj)
                    inst.save()
            for inst in frms:
                setattr(inst, self.inline_fk_name, self.old_obj)
                inst.save()
            
        else:
            return formset.save()
    def log_addition(self, request, object):
        super(ConfigurePreviewMixinAdmin, self).log_addition(request, object)
        # TODO: spostare la cancellazione alla fine dell'aggiornamento di tutti i formsets!
        if self.old_obj:
            object.delete() #  ho cancellato la preview
            object.id = self.old_obj.id # sostituisco l'originale con la preview
            setattr(object,self.preview_variable,False)
            object.save()
            self.curr_form.save_m2m() 
            
    def log_change(self, request, object, message):
        super(ConfigurePreviewMixinAdmin, self).log_change(request, object, message)
        # TODO: spostare la cancellazione alla fine dell'aggiornamento di tutti i formsets!
        if self.old_obj:
            object.delete() #  ho cancellato la preview
            object.id = self.old_obj.id # sostituisco l'originale con la preview
            setattr(object,self.preview_variable,False)
            object.save()
            self.curr_form.save_m2m()
            
    # Operazione da Mixin
    def redirect_for_other(self, request, obj, msg, add):
        opts = obj._meta
        if request.POST.has_key("_is_preview"):
            #self.message_user(request, msg + ' ' + (_("You may add another %s below with same customer.") % force_unicode(opts.verbose_name)))
            #potresti fare il redirect alla preview, ma devi pensare ad un modo per tornare indietro
            return HttpResponseRedirect("../%(id)s/?%(preview_var)s=%(id)s" % {'id' : obj.id, 'preview_var' : self.preview_variable})
        else:
            return super(ConfigurePreviewMixinAdmin, self).redirect_for_other(request, obj, msg, add)


