# -*- coding: utf-8 -*-
import logging

from django.conf import settings
from django.contrib.auth.models import User, Group, Permission
from django.core.exceptions import ImproperlyConfigured
from django.core.management import call_command
from django.core.serializers.json import Serializer
from django.core.urlresolvers import reverse
from django.contrib.contenttypes.models import ContentType
from django.http import HttpRequest
from django.test import TestCase, Client
from django.test.client import RequestFactory
from django.test.utils import override_settings
from django.utils.importlib import import_module

try:
    # Django <= 1.6 backwards compatibility
    from django.utils import simplejson as json
except ImportError:
    # Django >= 1.7
    import json

logger = logging.getLogger(__name__)


    
#################################################################################
# Base Tests: da spostare in fHCoreLibs o da rimpiazzare con test più seri che facciano anche concorrenza
#################################################################################


class TastypieLikeTestCase(TestCase):
    
    def assertHttpOK(self, resp):
        """
        Ensures the response is returning a HTTP 200.
        """
        return self.assertEqual(resp.status_code, 200)

    def assertValidJSON(self, data):
        """
        Given the provided ``data`` as a string, ensures that it is valid JSON &
        can be loaded properly.
        """
        # Just try the load. If it throws an exception, the test case will fail.
        #self.serializer.from_json(data)
        return json.loads(data)
        
    def assertValidJSONResponse(self, resp):
        """
        Given a ``HttpResponse`` coming back from using the ``client``, assert that
        you get back:

        * An HTTP 200
        * The correct content-type (``application/json``)
        * The content is valid JSON
        """
        self.assertHttpOK(resp)
        self.assertTrue(resp['Content-Type'].startswith('application/json'))
        self.assertValidJSON(resp.content)        

    def deserialize(self, resp):
        """
        Given a ``HttpResponse`` coming back from using the ``client``, this method
        checks the ``Content-Type`` header & attempts to deserialize the data based on
        that.

        It returns a Python datastructure (typically a ``dict``) of the serialized data.
        """
        return json.loads(resp.content)

    def serialize(self, data, format='application/json'):
        """
        Given a Python datastructure (typically a ``dict``) & a desired content-type,
        this method will return a serialized string of that data.
        """
        return json.dumps(data)


class SessionTestCase(TestCase):
    
    def setUp(self):
        # http://code.djangoproject.com/ticket/10899
        super(SessionTestCase, self).setUp()
        settings.SESSION_ENGINE = 'django.contrib.sessions.backends.file'
        engine = import_module(settings.SESSION_ENGINE)
        store = engine.SessionStore()
        store.save()
        self.client = Client()
        self.session = store
        self.client.cookies[settings.SESSION_COOKIE_NAME] = store.session_key
        #self.client.session = self.session
        self.serializer = Serializer()
        #self.fb_counter = 0
        
    def set_in_session(self, **kwargs):
        kwargs = kwargs or {}
        session = self.session
        for k,v in kwargs.iteritems():
            session[k] = v
        session.save()


class ClassBasedTestViews(TastypieLikeTestCase):

    def assertRedirectView(self, **kwargs):
        """
        @var url : the reverse url of the View
        @var next_url : the url to redirect_to
        @var status  : Status code, optional... default = 302 (Temporary) ... remember 301 for Permanent
        """
        url = kwargs.get('url',None)
        expected_status = kwargs.get('status',302)
        expected_next_url = kwargs.get('next_url',None)
        
        if not url:
            raise ImproperlyConfigured()
        new_response = self.client.get(url)
        if expected_status:
            self.assertEquals(expected_status, new_response.status_code)
        if expected_next_url:
            logger.debug('expected_next_url {0}'.format(expected_next_url))
            self.assertEquals(new_response['Location'], "http://testserver{0}".format(expected_next_url)) #fb_reverse(request, 'game:checkuser'))
        return new_response
    
    def assertTemplateResponseView(self, **kwargs):
        """
        @var url : the reverse url of the View
        @var template_name : the name of the Templte(the path)
        @var status  : Status code, optional... default = 200
        """
        url = kwargs.get('url',None)
        expected_status = kwargs.get('status',200)
        expected_template = kwargs.get('template_name',None)
        
        if not url:
            raise ImproperlyConfigured()
        new_response = self.client.get(url)
        if expected_status:
            self.assertEquals(expected_status, new_response.status_code)
        if expected_template:
            self.assertTemplateUsed(expected_template, new_response.status_code)
        return new_response

    def assertValidJSONResponseView(self, **kwargs):
        """
        @var url : the reverse url of the View
        @var status  : Status code, optional... default = 200
        @var method : method of the call... POST or GET(default) #TODO implement head ad others
        """
        url = kwargs.get('url',None)
        expected_status = kwargs.get('status',200)
        method = kwargs.get('method','GET')
        post_data = kwargs.get('post_data',{})
        
        if method == 'GET':
            json_response = self.client.get(url)
        elif method == 'POST':
            json_response = self.client.post(url,post_data)
        else:
            raise AssertionError("Methods accepted : POST,GET ")
        self.assertValidJSONResponse(json_response)
        return json_response



try:
    import requests
    import httplib
    import random
    from fbconsole import Client as FBClient
    # mandatory
    FB_APP_ID = settings.FB_APP_DATA.get('app_id')
    FB_APP_SECRET = settings.FB_APP_DATA.get('app_secret')
    FB_APP_SCOPE = ()


    class FacebookTestUtility(object):
        def __init__(self, appId, appSecret):
            self.appId = appId
            self.appSecret = appSecret
            self.host = "graph.facebook.com";
            self.testUsersUrl = "{0}/{1}/accounts/test-users".format(self.host, self.appId)
            self.fmtGet = "/{0}/accounts/test-users?access_token={0}|{1}&limit=50"
            self.fmtDel = "/{0}?method=delete&access_token={1}|{2}"
            
        def deleteTestUsers(self):
            usersPath = self.fmtGet.format(self.appId, self.appSecret)
            conn = httplib.HTTPSConnection(self.host)
            while usersPath:
                #print usersPath
                conn.request("GET", usersPath);
                users = json.loads(conn.getresponse().read());
                #print("Got {0} users.".format(len(users["data"])))
                if len(users["data"])==0:
                    #print("no more")
                    break
                for user in users["data"]:
                    #print("Deleting user {0}".format(user["id"]))
                    delPath = self.fmtDel.format(user["id"], self.appId, self.appSecret)
                    conn = httplib.HTTPSConnection(self.host)
                    conn.request("GET", delPath)
                    deleteResult = conn.getresponse().read()
                    if deleteResult is not "true":
                        logger.warning("delete failed, got {0}".format(deleteResult))
                #print(users)
                if not "paging" in users:
                    #print("paging exiting")
                    break
                paging = users["paging"];
                if not "next" in users["paging"]:
                    #print("next exiting")
                    break
                nextUrl = paging["next"];
                prefix = "https://{0}".format(self.host)
                if not nextUrl.startswith(prefix):
                    #print("Paging url does not start with {0}".format(prefix))
                    #print("no page exit")
                    break
                #usersPath = nextUrl[len(prefix):]
                #print( "Continuing next page at: {0}".format(usersPath) )
                
        def createTestUser(self, installed=True, full_name=None, permissions=["read_stream",]):
            access_token = "{0}|{1}".format(self.appId, self.appSecret)
            fb_params = {
                    "access_token": access_token,
                    "method": "POST",
                    "installed": installed,
                    "permissions": ",".join(permissions),
            }
            #logger.info("ACCESS TOKEN : %s" %(access_token,))
            fb_response = requests.get(
                "https://{0}".format(self.testUsersUrl),
                params=fb_params,
                verify=False)
            self.assertEqual(fb_response.status_code,200)
            #logger.info("FB RESPONSE : %s" %(fb_response.json(),))
                
            #assert fb_response.status_code==200
            #fb_json_response = json.loads(fb_response)
            #r contiene Email e Password
            return fb_response.json()#fb_json_response #
        
        def createTestUsers(self, cnt=20, **kwargs):
            for i in range(0,cnt):
                #logger.info("adding user {0}...".format(i))
                self.createTestUser(**kwargs)
                
        def listTestUsers(self):
            users = []
            usersPath = self.fmtGet.format(self.appId, self.appSecret)
            while usersPath:
                conn = httplib.HTTPSConnection(self.host)
                conn.request("GET", usersPath);
                json_data = json.loads(conn.getresponse().read())
                users.extend(json_data["data"])
                if not "paging" in json_data:
                    break
                paging = json_data["paging"];
                if not "next" in paging:
                    break
                nextUrl = paging["next"]
                prefix = "https://{0}".format(self.host)
                if not nextUrl.startswith(prefix):
                    #logger.warning("Paging url does not start with {0}".format(prefix))
                    break
                usersPath = nextUrl[len(prefix):]
            return users
        
        def authenticateUser(self, user_data):
            authenticated = None
            if user_data:
                if "access_token" in user_data.keys():
                    authenticated = FBClient(user_data["access_token"])
            return authenticated
        
        def authenticateWithRandomUser(self):
            ll = self.listTestUsers()
            user = random.choice(ll)
            return self.authenticateUser(user)

    
    class FacebookTestCase(TastypieLikeTestCase):
    
        def setUp(self):
            # http://code.djangoproject.com/ticket/10899
            super(FacebookTestCase, self).setUp()
            self.client = Client()
            self.fbTestUtility = FacebookTestUtility(FB_APP_ID, FB_APP_SECRET)
    
    class FacebookAndSessionTestCase(TastypieLikeTestCase):
    
        def setUp(self):
            # http://code.djangoproject.com/ticket/10899
            super(FacebookAndSessionTestCase, self).setUp()
            settings.SESSION_ENGINE = 'django.contrib.sessions.backends.file'
            engine = import_module(settings.SESSION_ENGINE)
            store = engine.SessionStore()
            store.save()
            self.client = Client()
            self.session = store
            self.client.cookies[settings.SESSION_COOKIE_NAME] = store.session_key
            #self.client.session = self.session
            self.serializer = Serializer()
            #self.fb_counter = 0
            self.fbTestUtility = FacebookTestUtility(FB_APP_ID, FB_APP_SECRET)
            
        def set_in_session(self, **kwargs):
            kwargs = kwargs or {}
            session = self.session
            for k,v in kwargs.iteritems():
                session[k] = v
            session.save()

except KeyError as ex:
    msg = u'facebook config error. Please check "FB_APP_DATA" on your settings'
    logger.exception(msg)
    #raise ImproperlyConfigured(msg)
except ImportError as ex:
    logger.exception("Import Error : requests, httplib e fbconsole required")
    pass
    #raise ImproperlyConfigured(msg)

    #raise ImproperlyConfigured("FH Facebook Views require the 'fbconsole' cots module")

