# -*- coding: utf-8 -*-
""" fHCoreLibs > Views > Base.py
    @author: Domenico Lamparelli <domenico@frankhood.it> 
    @author: Nicola Mosca <nico@frankhood.it>
    @author: Gaetano D'Onghia <g.donghia@frankhood.it>
"""
from __future__ import absolute_import, print_function, unicode_literals
from django import http
from django.core.context_processors import csrf
from django.core.paginator import Paginator
from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist, FieldError
from django.core.serializers.json import DjangoJSONEncoder
from django.core.urlresolvers import reverse
from django.http import HttpResponse, Http404, HttpResponseRedirect
from django.template.loader import render_to_string
from django.views.decorators.cache import never_cache
from django.views.generic.base import TemplateResponseMixin
from django.views.generic.detail import DetailView
from django.views.generic.list import ListView
from django.utils.decorators import method_decorator
from django.utils.encoding import smart_str
from django.utils.translation import ugettext as _
from django.utils.translation import ugettext_lazy as _l

from ..exceptions import RedirectException
from ..views.generic import fHCoreListView, fHCoreDetailView

try:
    import json
except ImportError:
    from django.utils import simplejson as json

class JSONResponseMixin(object):
    
    encoder = DjangoJSONEncoder
    content_type = 'application/json'
    
    def get_content_type(self):
        return self.content_type
    
    def render_to_response(self, context):
        "Returns a JSON response containing 'context' as payload"
        return self.get_json_response(self.convert_context_to_json(context))

    def get_json_response(self, content, **httpresponse_kwargs):
        "Construct an `HttpResponse` object."
        return http.HttpResponse(
            content,
            content_type=self.get_content_type(),
            **httpresponse_kwargs
        )

    def convert_context_to_json(self, context, **kwargs):
        "Convert the context dictionary into a JSON object"
        return json.dumps(context)

class JSONStructuredResponseMixin(JSONResponseMixin):

    success_msg = _("La procedura è andata a buon fine")
    failure_msg = _("Problemi nella procedura")
    check_success = False

    def get_check_success(self):
        return self.check_success
    
    def convert_context_to_json(self, context, **kwargs):
        kwargs = kwargs or {}
        kwargs.update(context)
        if self.get_check_success():
            kwargs.update({'msg':self.success_msg})
            context = self.data_valid(**kwargs)
        else:
            kwargs.update({'msg':self.failure_msg})
            context = self.data_invalid(**kwargs)
        json_context = super(JSONStructuredResponseMixin,self).convert_context_to_json(context, **kwargs)
        return json_context

    def add_success_metadata(self):
        return {}

    def add_failure_metadata(self):
        return {}
    
    def data_valid(self, **kwargs):
        msg = kwargs.get('msg',"")
        return {
            "status_code" :200,
            "message": {'text': msg, 'status': 'SUCCESS'},
            'jsonrpc': '2.0',
            'meta' : self.add_success_metadata()
        }

    def data_invalid(self, **kwargs):
        msg = kwargs.get('msg', "None")
        return {
            "status_code" :301,
            'message': {'text': msg, 'status': 'FAIL'},
            #'file_size': file.size, 
            'jsonrpc': '2.0',
            'meta' : self.add_failure_metadata()
        }

class JSONHijaxResponseMixin(TemplateResponseMixin):
    """ @TODO: Da astrarre per vari formati """
    
    xhr_template_name = None
    xhr_context_object_name = None
    xhr_content_type = 'application/json'
    
    def render_to_response(self, context):
        if self.request.is_ajax():
            return self.get_json_response(self.convert_context_to_json(context))
        return super(JSONHijaxResponseMixin,self).render_to_response(context)
    
    def get_xhr_template_name(self):
        return self.xhr_template_name or None
    
    def get_xhr_content_type(self):
        return self.xhr_content_type
    
    def get_template_names(self):
        template_names = super(JSONHijaxResponseMixin, self).get_template_names()
        if self.request.is_ajax():
            self.xhr_template_name = self.get_xhr_template_name()
            if self.xhr_template_name is None:
                raise ImproperlyConfigured("JSONHijaxResponseMixin requires a definition of ""'xhr_template_name''")
            template_names = [self.xhr_template_name] + template_names
        return template_names

    def get_json_response(self, content, **httpresponse_kwargs):
        return http.HttpResponse(
            content,
            content_type=self.get_xhr_content_type(),
            **httpresponse_kwargs
        )

    
    def convert_context_to_json(self, context, **kwargs):
        "Convert the context dictionary into a JSON object"
        return json.dumps(context)
    
    def get_context_object_name(self, object_list):
        """
        Get the name of the item to be used in the context.
        """
        if self.request.is_ajax() and self.xhr_context_object_name:
            self.context_object_name = self.xhr_context_object_name
            return self.xhr_context_object_name 
        elif self.context_object_name:
            return self.context_object_name
        elif hasattr(object_list, 'model'):
            return smart_str('%s_list' % object_list.model._meta.object_name.lower())
        else:
            return None


class HeadResponseMixin(object):
    
    def head(self, request, *args, **kwargs):
        response = HttpResponse()
        #response['Cache-Control'] = 'no-cache'
        return response


class JSONFormBaseMixin(object):
    valid_message = _("Grazie per averci contattato")
    invalid_message = _("Problemi nel salvataggio")
    form_not_valid_message = _("Controlla i dati nel form")
    form_object_name = 'form'
    
    def data_valid(self):
        #http://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html
        if hasattr(self,'object'): 
            if self.object:
                return { 'status':201, 'message': { 'text': self.valid_message, 'status': 'ok' }}
            else:
                return { 'status':304, 'message': { 'text': self.invalid_message, 'status': 'Error' }}
        else:
            return { 'status':201, 'message': { 'text': self.valid_message, 'status': 'ok' }}
            
    def data_invalid(self, form=None):
        if form:
            return { 'status':400, 'message': { 'text': self.form_not_valid_message, 
                                               'status': 'Form Error', 
                                               'errors' : form.errors}}
        else:
            return { 'status':304, 'message': { 'text': self.invalid_message, 
                                               'status': 'Data Error'}}
    def get_context_data(self, **kwargs):
        context = super(JSONFormBaseMixin, self).get_context_data(**kwargs)
        if self.request.is_ajax():
            _data_valid = kwargs.get('data_is_valid', True)
            if not _data_valid:
                if context.has_key('form'):
                    context = self.data_invalid(context['form'])
                else:
                    context = self.data_invalid()
            else:
                context = self.data_valid()
        return context
        
    def form_valid(self, form):
        if hasattr(self,'object'):
            self.object = form.save()
        if self.request.is_ajax(): 
            return self.render_to_response(self.get_context_data(**({'data_is_valid':True}))) 
        else:
            return HttpResponseRedirect(self.get_success_url())
         
    def form_invalid(self, form):
        return self.render_to_response(self.get_context_data(form=form, **({'data_is_valid':False}))) 

class JSONFormMixin(JSONFormBaseMixin, JSONResponseMixin):
    pass 

class JSONHijaxFormMixin(JSONFormBaseMixin, JSONHijaxResponseMixin):
    pass     

class SingleObjectIdAndSlugMixin(object):
    
    def get_object(self, queryset=None):
        """
        Returns the object the view is displaying.

        By default this requires `self.queryset` and a `pk` AND `slug` argument
        in the URLconf, but subclasses can override this to return any object.
        """
        # Use a custom queryset if provided; this is required for subclasses
        # like DateDetailView
        if queryset is None:
            queryset = self.get_queryset()

        # Next, try looking up by primary key.
        pk = self.kwargs.get('pk', None)
        slug = self.kwargs.get('slug', None)
        if pk is not None and slug is not None:
            queryset = queryset.filter(pk=pk)
            slug_field = self.get_slug_field()
            queryset = queryset.filter(**{slug_field: slug})
        elif slug is not None:
            slug_field = self.get_slug_field()
            queryset = queryset.filter(**{slug_field: slug}) 
        # If none of those are defined, it's an error.
        else:
            raise Http404((u"SingleObjectIdAndSlugMixin overrided by %s view must be called with "
                           u"an object pk AND a slug."
                           % self.__class__.__name__))
        try:
            obj = queryset.get()
        except ObjectDoesNotExist:
            raise Http404(_(u"No %(verbose_name)s found matching the query") %
                          {'verbose_name': queryset.model._meta.verbose_name})
        return obj
        