# -*- coding: utf-8 -*-
""" fHCoreLibs > VIEWS > Generic.py
    @author: Domenico Lamparelli <domenico@frankhood.it> 
    @author: Nicola Mosca <nico@frankhood.it>
    @author: Gaetano D'Onghia <g.donghia@frankhood.it>
"""

from django.core.exceptions import ImproperlyConfigured, FieldError
from django.core.urlresolvers import reverse
from django.http import Http404, HttpResponseRedirect
from django.views.generic.base import RedirectView
from django.views.generic.detail import DetailView
from django.views.generic.list import ListView

from ..exceptions import RedirectException



class CustomManagerMixin(object):
    """
    Permette di cambiare il manager di self.Model non modificando il comportamento di MultipleObjetMixin o di Single Object Mixin
    """
    
    model_manager = '_default_manager'
    
    def get_model_manager(self):
        return self.model_manager
    
    def get_queryset(self):
        """
        Get the list of items for this view. This must be an iterable, and may
        be a queryset (in which qs-specific behavior will be enabled).
        """
        if self.queryset is not None:
            queryset = self.queryset
            if hasattr(queryset, '_clone'):
                queryset = queryset._clone()
        elif self.model is not None:
            if hasattr(self.model,self.model_manager):
                try:
                    queryset = getattr(self.model,self.get_model_manager()).all()
                except AttributeError:
                    raise ImproperlyConfigured(u"'%s' model doesn't have '%s' manager"
                                       % (self.model._meta.verbose_name, self.model_manager))
            else:
                raise ImproperlyConfigured(u"'%s' must define '%s' manager"
                                       % (self.model.__name__, self.model_manager,))
        else:
            raise ImproperlyConfigured(u"'%s' must define 'queryset' or 'model'"
                                       % self.__class__.__name__)
        return queryset
    
class fHCoreDetailView(CustomManagerMixin, DetailView):
    template_name = None  
    model = None
    model_manager = '_default_manager'
    context_object_name = None
    
    def set_request_data(self, request):
        self._request_data = request.GET.copy()  #qui dentro ho il dizionario con i parametri della query
        self._request_data.update(self.kwargs.copy())
    
    def pre_process_view(self):
        pass
    
    def post_process_view(self, *args, **kwargs):
        pass
    
    def get(self, request, **kwargs):
        self.set_request_data(request)
        try:
            self.pre_process_view()
        except RedirectException,e:
            return HttpResponseRedirect(reverse(e.reverse_url, kwargs=e.kw)+ e.querystring)
        self.object = self.get_object()
        context = self.get_context_data(object=self.object)
        try:    
            self.post_process_view(**(context))
        except RedirectException,e:
            return HttpResponseRedirect(reverse(e.reverse_url, kwargs=e.kw)+ e.querystring)
        return self.render_to_response(context)
    

class fHCoreListView(CustomManagerMixin, ListView):
    template_name = None  
    allow_empty = True
    model = None
    model_manager = '_default_manager'
    context_object_name = None
    
    def set_request_data(self, request):
        self._request_data = request.GET.copy()  #qui dentro ho il dizionario con i parametri della query
        self._request_data.update(self.kwargs.copy())
    
    def pre_process_view(self):
        pass
    
    def post_process_view(self, *args, **kwargs):
        pass
    
    def get(self, request, *args, **kwargs):
        self.set_request_data(request)
        try:
            self.pre_process_view()
        except RedirectException,e:
            return HttpResponseRedirect(reverse(e.reverse_url, kwargs=e.kw)+ e.querystring)
        self.object_list = self.get_queryset()
        allow_empty = self.get_allow_empty()
        if not allow_empty and len(self.object_list) == 0:
            raise Http404(_(u"Empty list and '%(class_name)s.allow_empty' is False.")
                          % {'class_name': self.__class__.__name__})
        context = self.get_context_data(object_list=self.object_list)
        try:    
            self.post_process_view(**(context))
        except RedirectException,e:
            return HttpResponseRedirect(reverse(e.reverse_url, kwargs=e.kw)+ e.querystring)
        return self.render_to_response(context)
    

class ForeignKeyMixin(object):
    fk_model = None
    fk_model_manager = None
    fk_variable = None
    fk_variable_slug_field = 'slug'
    fk_slug = ''
    fk_object_name = 'categories'
    
    def get_related_queryset(self,qs,fk_slug):
        qs = qs.filter(**{'%(fk_var)s__%(fk_var_slug)s' %({'fk_var': self.fk_variable,'fk_var_slug':self.fk_variable_slug_field}) : fk_slug})
        return qs
    
    def get_queryset(self):
        qs = super(ForeignKeyMixin,self).get_queryset()
        if self.fk_model:
            fk_topic = self._request_data[self.fk_slug] if self._request_data.has_key(self.fk_slug) else ''
            if fk_topic:
                qs = self.get_related_queryset(qs,fk_topic)
        return qs
    
    def get_context_data(self, *args, **kwargs):
        context_data = super(ForeignKeyMixin,self).get_context_data(*args, **kwargs)
        context_data[self.fk_object_name] = getattr(self.fk_model,self.fk_model_manager).all() if self.fk_model_manager else self.fk_model._default_manager.all()
        return context_data
  
    
class QueryMixin(object):
    filter_dict = {}
    order_dict = {}
    
    def build_query(self):
        filter_kwargs = {}
        for k,v in self._request_data.iteritems():
            try:
                if v <> 'all':
                    filter_kwargs.update({self.filter_dict[k]:v})
                self._request_data.pop(k)
            except KeyError:
                pass
                #non faccio RAISE xke potrebbero servire delle variabili presenti in self._request_data tipo page
                #raise Http404(_(u"Key error in processing '%(class_name)s'")
                #      % {'class_name': self.__class__.__name__})
        return filter_kwargs    
    
    def order_queryset(self,qs):
        for k,v in self._request_data.iteritems():
            if v in ['asc', 'desc']:
                try:
                    qs = qs.order_by(self.order_dict[k]) if v == 'asc' else qs.order_by('-'+self.order_dict[k])
                    self._request_data.pop(k)
                except KeyError:
                    pass 
                except FieldError:
                    raise Http404(_(u"Cannot order for '%(order_field)s'")
                          % {'order_field': k})
        return qs
    
    def get_queryset(self):
        queryset = super(QueryMixin,self).get_queryset()
        queryset = queryset.filter(**(self.build_query()))
        queryset = self.order_queryset(queryset)
        return queryset   

class fHCoreListWithFKView(ForeignKeyMixin, fHCoreListView):
    """
    ListView in which self.model in sorrelated with a FK(Category, Topic or others)
    """
    
class fHCoreAdvancedListView(QueryMixin, fHCoreListView):
    """
    Render some list of objects, set by `self.model` or `self.queryset`.
    `self.queryset` can actually be any iterable of items, not just a queryset.
    This queryset is filtered and ordered tnx to dicts self.filter_dict and self.order_dict
    """ 

    
class ReverseRedirectView(RedirectView):
    '''
    Override di RedirectView x implementazioni con reverse, querystring, e kwargs
    USAGE:
    url(r'^toyota/$',never_cache(ReverseRedirectView.as_view(reverse_url='service-coupon-view')),kwargs={'slug':'toyota','querystring'='ciao=1&r=2','admin_view':False},name='old-service-coupon-toyota-view'),
    '''
    reverse_url = None

    
    def pre_process_view(self,*args,**kwargs):
        if self.reverse_url:
            raise RedirectException(self.reverse_url,**kwargs)
    
    def get(self, request, *args, **kwargs):
        try:
            self.pre_process_view(*args,**kwargs)
        except RedirectException,e:
            return HttpResponseRedirect(reverse(e.reverse_url, kwargs=e.kw)+ e.querystring)
        return super(ReverseRedirectView,self).get(request, *args, **kwargs)