# -*- coding: utf-8 -*-
""" frankhood > fHCoreLibs > Views > preview_views.py by Frankhood
    @author: Frankhood Business Solutions
"""
from django.core.exceptions import ObjectDoesNotExist, ImproperlyConfigured
from django.core.urlresolvers import reverse
from django.http import HttpResponse, HttpResponseBadRequest, Http404, HttpResponseForbidden
from django.http import HttpResponseRedirect
from django.template.context import RequestContext
from django.utils.translation import get_language, ugettext as _
from django.views.generic import ListView
from django.views.generic.detail import DetailView
from ..exceptions import RedirectException


class PreviewDetailView(DetailView):
    
    model = None
    manager = 'published_objects'
    preview_manager = 'not_preview_pub_objects'
    preview_variable = 'is_preview'
    slug_field = 'slug' 
    context_object_name = 'item'
    is_lang_unique = True
    has_preview_permission = False
    
    def get_object(self, queryset=None):
        """
        The difference with Detail View is that you have to take the object with
         'id' or with the 'unique_togethers'...
        Returns the object the view is displaying.

        By default this requires `self.queryset` and a `pk` or `slug` argument
        in the URLconf, but subclasses can override this to return any object.
        """
        # Use a custom queryset if provided; this is required for subclasses
        # like DateDetailView
        if queryset is None:
            queryset = self.get_queryset()

        # Next, try looking up by primary key.
        
        pk = self._request_data['pk'] if self._request_data.has_key('pk') else None
        slug = self._request_data[self.slug_field] if self._request_data.has_key(self.slug_field) else None
        lang = self._request_data['lang'] if self._request_data.has_key('lang') else get_language()
        is_preview_request = self._request_data['is_preview'] if self._request_data.has_key('is_preview') else False
        
        if pk is not None:
            queryset = queryset.filter(pk=pk)

        # Next, try looking up by slug.
        elif slug is not None:
            slug_field = self.get_slug_field()
            if is_preview_request and (not self.has_preview_permission) :
                raise Http404(_(u"Not allowed"))
            queryset = queryset.filter(**{slug_field: slug, self.preview_variable: is_preview_request})
            if self.is_lang_unique:
                queryset = queryset.filter(**{'lang' : lang})

        # If none of those are defined, it's an error.
        else:
            raise AttributeError(u"PreviewDetailView  %s must be called with "
                                 u"either an object pk or a slug with optional lang and is_preview flag."
                                 % self.__class__.__name__)

        try:
            obj = queryset.get()
        except ObjectDoesNotExist:
            raise Http404(_(u"No %(verbose_name)s found matching the query") %
                          {'verbose_name': queryset.model._meta.verbose_name})
        return obj
    
    def get_queryset(self):
        """
        Get the queryset to look an object up against. May not be called if
        `get_object` is overridden.
        """
        if self.queryset is None:
            if self.model:
                try:
                    if self.has_preview_permission:
                        return getattr(self.model, self.manager).all()
                    else: 
                        return getattr(self.model, self.preview_manager).all()
                except AttributeError:
                    raise Http404(_(u"No %(verbose_name)s found matching the query") %
                          {'verbose_name': self.model._meta.verbose_name})
            else:
                raise ImproperlyConfigured(u"%(cls)s is missing a queryset. Define "
                                           u"%(cls)s.model, %(cls)s.queryset, or override "
                                           u"%(cls)s.get_object()." % {
                                                'cls': self.__class__.__name__
                                        })
        return self.queryset._clone()
    
    def set_request_data(self, request):
        self._request_data = request.GET.copy()
        self._request_data.update(self.kwargs.copy())
    
    def pre_process_view(self):
        pass
    
    def post_process_view(self, *args, **kwargs):
        pass
    
    def get(self, request, *args, **kwargs):
        self.set_request_data(request)
        try:
            self.pre_process_view()
        except RedirectException,e:
            return HttpResponseRedirect(reverse(e.reverse_url, kwargs=e.kw)+ e.querystring)
        self.object = self.get_object()
        context = self.get_context_data(object=self.object)
        try:
            self.post_process_view()
        except RedirectException,e:
            return HttpResponseRedirect(reverse(e.reverse_url, kwargs=e.kw)+ e.querystring)
        return self.render_to_response(context)

    
class PreviewDetailWithFkView(PreviewDetailView):
    fk_slug_field = 'fk_slug'
    fk_variable = None
    fk_variable_slug = 'slug'
    
    def get_related_queryset(self,qs,fk):
        qs = qs.filter(**{'%(fk_var)s__%(fk_var_slug)s' %({'fk_var': self.fk_variable,'fk_var_slug':self.fk_variable_slug}) :fk})
        return qs
    
    def get_queryset(self):
        qs = super(PreviewDetailWithFkView,self).get_queryset()
        if self.fk_variable:
            fk = self._request_data[self.fk_slug_field] if self._request_data.has_key(self.fk_slug_field) else None
            if fk:
                qs = self.get_related_queryset(qs,fk)
            else:
                if not self.has_preview_permission:
                    raise Http404(_(u"No %(verbose_name)s found matching the query") %
                                  {'verbose_name': qs.model._meta.verbose_name})
        else:
            raise ImproperlyConfigured(_(u"Insert the FK variable of model %(verbose_name)s in the view ") %
                          {'verbose_name': qs.model._meta.verbose_name})
        return qs

        

class PreviewListView(ListView):
    
    model = None
    model_manager = 'not_preview_pub_objects'
    model_preview_manager = 'published_objects' #
    container_model_manager = 'not_preview_pub_objects'
    container_model_preview_manager = 'published_objects'
    allow_empty = True
    container_model = None
    slug_field = 'slug'
    preview_variable = 'is_preview'
    context_object_name = 'entries'
    context_container_name = 'item'
    is_lang_unique = True
    fk_variable = None # if you have a items in an entry with ForeignKey, you have to insert this FK field here
    is_preview = False
    has_preview_permission = False
    
    
    def get_queryset(self):
        """
        Get the list of items for this view. This must be an interable, and may
        be a queryset (in which qs-specific behavior will be enabled).
        """
        if self.queryset is not None:
            queryset = self.queryset
            if hasattr(queryset, '_clone'):
                queryset = queryset._clone()
        elif self.model is not None:
            try:
                return getattr(self.model, self.model_manager).all()
            except AttributeError:
                return self.model._default_manager.all()
        else:
            raise ImproperlyConfigured(u"'%s' must define 'queryset' or 'model'"
                                       % self.__class__.__name__)
        return queryset

    def get_slug_field(self):
        """
        Get the name of a slug field to be used to look up by slug.
        """
        return self.slug_field

    def get_objects(self):
        """
        La funzione restituisce una tupla con 2 oggetti : real_obj e prev_obj
        Prendo l'oggetto di classe CONTAINER_MODE in base alla PK(cond.1A) o agli unique_together(cond.1B) 
        di cui fa parte lo slug_field, ma se l'oggetto è preview(cond.2) verifico 
        se sto chiedendo la preview e se ho i permessi per farlo e nel caso 
        mi prendo, se c'è, anche l'oggetto reale che sto modificando, il quale mi 
        permette di vedere la lista degli item associati all'oggetto originale 
        altrimenti se l'oggetto non è preview(cond.3), prendo l'oggetto reale e l'oggetto 
        preview in realtà sarà l'oggetto reale
            
        """
        
        pk = self._request_data['pk'] if self._request_data.has_key('pk') else None
        slug = self._request_data[self.slug_field] if self._request_data.has_key(self.slug_field) else None
        lang = self._request_data['lang'] if self._request_data.has_key('lang') else get_language()
        is_preview_request = self._request_data['is_preview'] if self._request_data.has_key('is_preview') else False
        
        # Use a custom queryset if provided; this is required for subclasses
        # like DateDetailView
        if self.queryset is not None:
            queryset = self.queryset
        try:
            if is_preview_request and self.has_preview_permission:
                queryset = getattr(self.container_model,self.container_model_preview_manager).all()
            else:
                queryset = getattr(self.container_model, self.container_model_manager).all()
        except:
            raise AttributeError(u"PreviewListView  %s must be called after outlining"
                                 u" the container model indicated in the FK field in self.model " 
                                 % self.__class__.__name__)

        # Next, try looking up by primary key.
        if pk is not None:
            #cond.1A
            qs_cloned = queryset._clone()
            queryset = queryset.filter(pk=pk)
            try:
                obj= queryset.get()
            except ObjectDoesNotExist:
                raise Http404(_(u"%s Entry not published.." % self.container_model._meta.verbose_name)) 
            is_preview_obj = getattr(obj,self.preview_variable)
            prev_obj = obj
            if is_preview_obj:
                #cond.2
                if is_preview_request and self.has_preview_permission:
                    qs_cloned = qs_cloned.filter(**{self.slug_field:getattr(obj,self.slug_field) ,self.preview_variable: False})
                    try:
                        real_obj = qs_cloned.get(**{'lang': obj.lang}) if self.is_lang_unique else qs_cloned.get() 
                    except ObjectDoesNotExist:
                        real_obj = None
                else:
                    raise Http404(_(u"You don't have Preview permission..Use Admin Interface")) 
            else:
                #cond.3
                # non utilizzata in questo primo progetto e quindi sempre non autorizzato con l'id
                if self.has_preview_permission:
                    real_obj = obj
                else:
                    raise Http404(_(u"You don't have Preview permission..Use Admin Interface")) 
        # Next, try looking up by slug.
       
        elif slug is not None:
            #cond.1B
            slug_field = self.get_slug_field()
            queryset = queryset.filter(**{slug_field: slug})
            if self.is_lang_unique:
                queryset = queryset.filter(**{'lang' : lang})
            try:
                real_obj = queryset.get(**{self.preview_variable:False})
            except ObjectDoesNotExist:
                real_obj = None
            if is_preview_request:
                #cond.2
                if not self.has_preview_permission:
                    # non utilizzato in questo primo progetto... bisognerebbe prevederlo dall'url
                    raise Http404(_(u"You don't have Preview permission..Use Admin Interface"))
                try:
                    prev_obj = queryset.get(**{self.preview_variable:True})
                except ObjectDoesNotExist:
                    raise Http404(_(u"No %(verbose_name)s found matching the query, with or without preview") %
                              {'verbose_name': queryset.model._meta.verbose_name})
            else:
                #cond.3
                prev_obj = real_obj
            # If none of those are defined, it's an error.
        else:
            raise AttributeError(u"PreviewListView  %s must be called with "
                                 u"either an object pk or a slug with optional lang and is_preview flag."
                                 % self.__class__.__name__)
        if not (real_obj or prev_obj):
            raise Http404(_(u"No %(verbose_name)s found matching the query, with or without preview") %
                          {'verbose_name': queryset.model._meta.verbose_name})
        return (real_obj,prev_obj)
    
    def set_request_data(self, request):
        self._request_data = request.GET.copy()
        self._request_data.update(self.kwargs.copy())
    
    def pre_process_view(self):
        pass
    
    def post_process_view(self, *args, **kwargs):
        pass
    
    def get(self, request, *args, **kwargs):
        self.set_request_data(request)
        try:
            self.pre_process_view()
        except RedirectException,e:
            return HttpResponseRedirect(reverse(e.reverse_url, kwargs=e.kw)+ e.querystring)
        (real_obj, preview_obj) = self.get_objects()
        self.container_object = preview_obj
        if real_obj:
            self.object_list = self.get_queryset()
            try:
                self.object_list = self.object_list.filter(**{self.fk_variable : real_obj}) 
            except:
                raise Http404(_(u"FK_variable not insered in the View of %(verbose_name)s ") %
                              {'verbose_name': self.object_list.model._meta.verbose_name})
        else:
            self.object_list = []
        allow_empty = self.get_allow_empty()
        if not allow_empty and len(self.object_list) == 0:
            raise Http404(_(u"Empty list and '%(class_name)s.allow_empty' is False.")
                          % {'class_name': self.__class__.__name__})
        context_kw = {self.context_container_name: self.container_object}
        context_data = self.get_context_data(object_list=self.object_list,**(context_kw))
        try:    
            self.post_process_view(**(context_data))
        except RedirectException,e:
            return HttpResponseRedirect(reverse(e.reverse_url, kwargs=e.kw)+ e.querystring)
        return self.render_to_response(context_data)
    
        

class PreviewListOrDetailView(PreviewListView):
    detail_redirect_view = None
    list_redirect_view = None
    redirect_to_detail_required = False
    redirect_to_list_required = False
    
    def pre_process_view(self):
        pk = self._request_data['pk'] if self._request_data.has_key('pk') else None
        slug = self._request_data['slug'] if self._request_data.has_key('slug') else None
        if (not pk) and (not slug):
            if self.redirect_to_list_required and self.list_redirect_view:
                try:
                    first_list_item = getattr(self.container_model,self.container_model_manager).all()[0]
                except IndexError:
                    raise Http404(_(u"No %(verbose_name)s found matching the query, with or without preview") %
                          {'verbose_name': self.container_model._meta.verbose_name})
                kw={'slug':getattr(first_list_item,self.slug_field)}
                raise RedirectException(self.list_redirect_view,**(kw))
    
    def post_process_view(self, *args, **kwargs):
        if self.object_list.count()==1 and self.redirect_to_detail_required:
            if not self.has_preview_permission:
                kw={'slug':self.object_list[0].slug, 'cat_slug':self.container_object.slug}
                raise RedirectException(self.detail_redirect_view,**(kw))
            else:
                kw={'pk':self.object_list[0].pk, 'is_admin_view':True}
                raise RedirectException(self.detail_redirect_view,**(kw))

class FilteredListMixin(object):
    
    fk_topic = ''
    fk_topic_model = None
    fk_topic_manager = 'not_preview_pub_objects'
    fk_topic_preview_manager = 'published_objects' 
    fk_topic_variable = None
    fk_topic_variable_slug = 'slug'
    fk_slug = ''
    is_preview = False
    has_preview_permission = False
    
    def get_related_queryset(self,qs,fk_slug):
        qs = qs.filter(**{'%(fk_var)s__%(fk_var_slug)s' %({'fk_var': self.fk_topic_variable,'fk_var_slug':self.fk_topic_variable_slug}) :fk_slug})
        return qs
    
    def get_queryset(self):
        qs = super(FilteredListMixin,self).get_queryset()
        if self.fk_topic_variable:
            self.fk_topic = self._request_data[self.fk_slug] if self._request_data.has_key(self.fk_slug) else ''
            if self.fk_topic:
                qs = self.get_related_queryset(qs,self.fk_topic)
        return qs
    
    def get_context_data(self, *args, **kwargs):
        context_data = super(FilteredListMixin,self).get_context_data(*args, **kwargs)
        is_preview_request = self._request_data['is_preview'] if self._request_data.has_key('is_preview') else False
        try:
            if self.has_preview_permission and is_preview_request and self.fk_topic:
                context_data['topics']= getattr(self.fk_topic_model,self.fk_topic_preview_manager).all()
            else:
                context_data['topics']= getattr(self.fk_topic_model,self.fk_topic_manager).all()
        except AttributeError:
            context_data['topics'] = []
        return context_data
    
