# -*- coding: utf-8 -*-
""" CORE MODULE > WIDGETS
    @author: Domenico Lamparelli <domenico@frankhood.it> 
    @author: Nicola Mosca <nico@frankhood.it>
    @author: Gaetano D'Onghia <g.donghia@frankhood.it>
"""
import time
import datetime
import re
from os import path
    


from django import forms
from django.conf import settings
from django.contrib.admin.widgets import AdminFileWidget
from django.db import models
from django.forms import fields, ValidationError
from django.forms.fields import EMPTY_VALUES
from django.forms.util import flatatt
from django.forms.widgets import Widget, Select, Input
from django.template import Template, Context
from django.template.loader import render_to_string
from django.utils import datetime_safe
from django.utils.dates import MONTHS_3
from django.utils.encoding import force_unicode, smart_unicode
from django.utils.formats import get_format
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _



empty_magic=object()


    
RE_DATE = re.compile(r'(\d{4})-(\d\d?)-(\d\d?)$')

class SelectDateWidget(Widget):
    """
    A Widget that splits date input into three <select> boxes.

    This also serves as an example of a Widget that has more than one HTML
    element and hence implements value_from_datadict.
    """
    none_value = (0, '---')
    month_field = '%s_month'
    day_field = '%s_day'
    year_field = '%s_year'

    def __init__(self, attrs=None, years=None, required=True):
        # years is an optional list/tuple of years to use in the "year" select box.
        self.attrs = attrs or {}
        self.required = required
        if years:
            self.years = years
        else:
            this_year = datetime.date.today().year
            self.years = range(this_year, this_year+10)

    def render(self, name, value, attrs=None):
        try:
            year_val, month_val, day_val = value.year, value.month, value.day
        except AttributeError:
            year_val = month_val = day_val = None
            if isinstance(value, basestring):
                if settings.USE_L10N:
                    try:
                        input_format = get_format('DATE_INPUT_FORMATS')[0]
                        # Python 2.4 compatibility:
                        #     v = datetime.datetime.strptime(value, input_format)
                        # would be clearer, but datetime.strptime was added in 
                        # Python 2.5
                        v = datetime.datetime(*(time.strptime(value, input_format)[0:6]))
                        year_val, month_val, day_val = v.year, v.month, v.day
                    except ValueError:
                        pass
                else:
                    match = RE_DATE.match(value)
                    if match:
                        year_val, month_val, day_val = [int(v) for v in match.groups()]
        choices = [(i, i) for i in self.years]
        year_html = self.create_select(name, self.year_field, value, year_val, choices)
        choices = MONTHS_3.items()
        month_html = self.create_select(name, self.month_field, value, month_val, choices)
        choices = [(i, i) for i in range(1, 32)]
        day_html = self.create_select(name, self.day_field, value, day_val,  choices)

        format = get_format('DATE_FORMAT')
        escaped = False
        output = []
        for char in format:
            if escaped:
                escaped = False
            elif char == '\\':
                escaped = True
            elif char in 'Yy':
                output.append(year_html)
            elif char in 'bFMmNn':
                output.append(month_html)
            elif char in 'dj':
                output.append(day_html)
        return mark_safe(u'\n'.join(output))

    def id_for_label(self, id_):
        return '%s_month' % id_
    id_for_label = classmethod(id_for_label)

    def value_from_datadict(self, data, files, name):
        y = data.get(self.year_field % name)
        m = data.get(self.month_field % name)
        d = data.get(self.day_field % name)
        if y == m == d == "0":
            return None
        if y and m and d:
            if settings.USE_L10N:
                input_format = get_format('DATE_INPUT_FORMATS')[0]
                try:
                    date_value = datetime.date(int(y), int(m), int(d))
                except ValueError:
                    pass
                else:
                    date_value = datetime_safe.new_date(date_value)
                    return date_value.strftime(input_format)
            else:
                return '%s-%s-%s' % (y, m, d)
        return data.get(name, None)

    def create_select(self, name, field, value, val, choices):
        if 'id' in self.attrs:
            id_ = self.attrs['id']
        else:
            id_ = 'id_%s' % name
        if not (self.required and val):
            choices.insert(0, self.none_value)
        local_attrs = self.build_attrs(id=field % id_)
        s = Select(choices=choices)
        select_html = s.render(field % name, val, local_attrs)
        return select_html

class AutocompleteInput(Input):
    input_type = 'text'   
    
    def __init__(self, view_url='/autocomplete/', attrs=None ): 
        self.ac_view_url = view_url
        super(AutocompleteInput,self).__init__(attrs)

    def _media(self):
        raw_media = forms.Media(css={'all': (settings.STATIC_URL+'fhcore/common/css/ui-lightness/jquery-ui-1.8.20.custom.css',
                                             settings.STATIC_URL+'fhcore/admin/css/autocomplete_for_django_admin.css')},
                                js=(settings.STATIC_URL+'fhcore/common/js/jquery-1.7.2.min.js', 
                                    settings.STATIC_URL+'fhcore/common/js/jquery-ui-1.8.20.custom.min.js'),
                                )
        return raw_media

    media = property(_media)

    def acomp_js_customize(self, field_id):
        f_path = 'admin/js/autocomplete_js.js'
        #os.path.join(settings.MEDIA_URL, 'admin', 'js', 'autocomplete_js.js')
        c = Context({ 'FIELD_ID': field_id, #field id
                      'VIEW_URL': self.ac_view_url
                    })        
        #f_string = ''.join(list(open(f_path, 'r')))
        #t = Template(f_string) 
        #return t.render(c)       
        return render_to_string(f_path,c )
        

    def render(self, name, value, attrs=None):
        id = attrs.get('id')
        midrender = super(AutocompleteInput,self).render(name, value, attrs)                
        acjs = '<script type="text/javascript">' + self.acomp_js_customize(id) + '</script>'
        endrender = midrender + acjs
        return mark_safe(endrender)


class AutocompleteWithHiddenInput(AutocompleteInput):
    """
    Simile a AutocompleteInput ma aggiunge un campo che sostituisce il campo principale
    La motivazione sta nel fatto che la presentazione deve apparire xke da piui informazioni del campo sul quale si applica:
    basti pensare ad una foreign key: il valore sarebbe un numero(l'ID)"""
    
    help_string = '_helper'
    
    def acomp_js_customize(self, field_id):
        f_path = 'admin/js/autocomplete_with_hidden.js'
        c = Context({ 'FIELD_ID': field_id, #field id
                      'VIEW_URL': self.ac_view_url,
                      'HELP_STRING' : self.help_string
                    })        
        return render_to_string(f_path,c )
        

    def render(self, name, value, attrs=None):
        if value is None:
            value = ''
        id = attrs.get('id')
        final_attrs = self.build_attrs(attrs, type='hidden', name=name)
        helper_attrs=attrs
        helper_attrs['id']+=self.help_string
        final_helper_attrs = self.build_attrs(helper_attrs, type=self.input_type, name=name+self.help_string)
        if value != '':
            # Only add the 'value' attribute if a value is non-empty.
            final_attrs['value'] = force_unicode(self._format_value(value))
        midrender = mark_safe(u'<input%s /><input%s />' % (flatatt(final_attrs),flatatt(final_helper_attrs)))
        acjs = '<script type="text/javascript">' + self.acomp_js_customize(id) + '</script>'
        endrender = midrender + acjs
        return mark_safe(endrender)


class AutocompleteWithFilterInput(Input):
     
    input_type = 'text'   
    help_string = '_helper'
    
    def __init__(self, view_url='/autocomplete_ctype/', c_type_id='id_content_type',  attrs=None ): 
        self.ac_view_url = view_url
        self.c_type_id = c_type_id
        super(AutocompleteWithFilterInput,self).__init__(attrs)

    def _media(self):
        raw_media = forms.Media(css={'all': (settings.STATIC_URL+'fhcore/common/css/ui-lightness/jquery-ui-1.8.20.custom.css',
                                             settings.STATIC_URL+'fhcore/admin/css/autocomplete_for_django_admin.css')},
                                js=(settings.STATIC_URL+'fhcore/common/js/jquery-1.7.2.min.js', 
                                    settings.STATIC_URL+'fhcore/common/js/jquery-ui-1.8.20.custom.min.js'),
                                )
        return raw_media

    media = property(_media)

    def acomp_js_customize(self, field_id):
        f_path = 'admin/js/autocomplete_with_filter.js'
        #os.path.join(settings.MEDIA_URL, 'admin', 'js', 'autocomplete_js.js')
        c = Context({ 'FILTER_ID': self.c_type_id, 
                      'FIELD_ID': field_id, #field id
                      'VIEW_URL': self.ac_view_url,
                      'HELP_STRING' : self.help_string
                    })        
        f_string = ''.join(list(open(f_path, 'r')))
        t = Template(f_string)        
        return t.render(c)

    def render(self, name, value, attrs=None):
        if value is None:
            value = ''
        id = attrs.get('id')
        final_attrs = self.build_attrs(attrs, type='hidden', name=name)
        helper_attrs=attrs
        helper_attrs['id']+=self.help_string
        final_helper_attrs = self.build_attrs(helper_attrs, type=self.input_type, name=name+self.help_string)
        if value != '':
            # Only add the 'value' attribute if a value is non-empty.
            final_attrs['value'] = force_unicode(self._format_value(value))
        midrender = mark_safe(u'<input%s /><input%s />' % (flatatt(final_attrs),flatatt(final_helper_attrs)))
        acjs = '<script type="text/javascript">' + self.acomp_js_customize(id) + '</script>'
        endrender = midrender + acjs
        return mark_safe(endrender)


class TextAndHiddenInput(Input):
     
    input_type = 'text'   
    help_string = '_helper'
    
    def __init__(self, input_class='object_id_gen', attrs=None ): 
        self.input_class = input_class
        super(TextAndHiddenInput,self).__init__(attrs)

    def render(self, name, value, attrs=None):
        if value is None:
            value = ''
        id = attrs.get('id')
        input_attrs = self.build_attrs(attrs, type='hidden', name=name)
        input_attrs['class'] = self.input_class
        helper_attrs = attrs
        helper_attrs['id']+=self.help_string
        helper_attrs['class'] = self.input_class + self.help_string
        final_helper_attrs = self.build_attrs(helper_attrs, type=self.input_type, name=name+self.help_string)
        if value != '':
            # Only add the 'value' attribute if a value is non-empty.
            input_attrs['value'] = force_unicode(self._format_value(value))
        midrender = mark_safe(u'<input%s /><input%s />' % (flatatt(input_attrs),flatatt(final_helper_attrs)))
        #acjs = '<script type="text/javascript">' + self.acomp_js_customize(id) + '</script>'
        acjs = ''
        endrender = midrender + acjs
        return mark_safe(endrender)
                 

                 
#===============================================================================
# WIDGETS
# - Html5EmailInput
# - Html5URLInput
# - ReadOnlyWidget
#===============================================================================

class Html5EmailInput(Input): 
    '''
        usage: email = forms.CharField(widget=Html5EmailInput())
    '''
    input_type = 'email'
 
class Html5URLInput(Input): 
    '''
        usage: url = forms.CharField(widget=Html5URLInput())
    '''
    input_type = 'url'
    
    
class ReadOnlyWidget(forms.Widget):
    '''
    http://djangosnippets.org/snippets/1682/
    Usage1: foo_field.widget=ReadOnlyWidget(mystring)
    Usage2: ReadOnlyWidget(myint, my_display_string)
    Usage3: ReadOnylWidget(form=myform, field_name='foo')
    '''
    def __init__(self, original_value=empty_magic, display_value=None, form=None, name=None):
        if original_value is empty_magic:
            assert form and name
            field=form.fields[name]
            original_value = form.initial.get(name, field.initial)
            if callable(original_value):
                original_value = original_value()
            if isinstance(field, forms.ChoiceField):
                for v, display in field.choices:
                    if v==original_value:
                        display_value=display
                        break
        else:
            assert not (form or name)
        if display_value is None:
            if original_value is None:
                display_value=u''
            else:
                display_value=original_value
        if isinstance(original_value, models.Model):
            original_value=original_value.pk
        self.original_value = original_value
        self.display_value = display_value

        super(ReadOnlyWidget, self).__init__()

    def render(self, name, value, attrs=None):
        if self.display_value is not None:
            if not isinstance(self.display_value, basestring):
                return unicode(self.display_value)
            return self.display_value
        return unicode(self.original_value)

    def value_from_datadict(self, data, files, name):
        return self.original_value
#===============================================================================
# FIELDS
#===============================================================================
class HexColorField(fields.Field):
    
    default_error_messages = {
        'hex_error': u'This is an invalid color code. It must be a html hex color code e.g. #000000'
    }

    def clean(self, value):
        
        super(HexColorField, self).clean(value)
        
        if value in fields.EMPTY_VALUES:
            return u''
        
        value = smart_unicode(value)
        value_length = len(value)
        
        if value_length != 7 or not re.match('^\#([a-fA-F0-9]{6}|[a-fA-F0-9]{3})$', value):
            raise ValidationError(self.error_messages['hex_error'])
        
        return value

    def widget_attrs(self, widget):
        if isinstance(widget, (fields.TextInput)):
            return {'maxlength': str(7)}

#===============================================================================

class RangeWidget(forms.MultiWidget):
    '''http://djangosnippets.org/snippets/1104/'''
    def __init__(self, widget, *args, **kwargs):
        widgets = (widget, widget)

        super(RangeWidget, self).__init__(widgets=widgets, *args, **kwargs)

    def decompress(self, value):
        return value

    def format_output(self, rendered_widgets):
        widget_context = {'min': rendered_widgets[0], 'max': rendered_widgets[1],}
        return render_to_string('widgets/range_widget.html', widget_context)


class RangeField(forms.MultiValueField):
    '''http://djangosnippets.org/snippets/1104/'''
    
    default_error_messages = {
        'invalid_start': _(u'Enter a valid start value.'),
        'invalid_end': _(u'Enter a valid end value.'),
    }

    def __init__(self, field_class, widget=forms.TextInput, *args, **kwargs):
        if not 'initial' in kwargs:
            kwargs['initial'] = ['','']

        fields = (field_class(), field_class())

        super(RangeField, self).__init__(
                fields=fields,
                widget=RangeWidget(widget),
                *args, **kwargs
                )

    def compress(self, data_list):
        if data_list:
            return [self.fields[0].clean(data_list[0]),self.fields[1].clean(data_list[1])]

        return None

#===============================================================================
# COLOR WIDGET AND FIELD
#===============================================================================
    
COLORPICKER_COLORS = getattr(settings, 'COLORPICKER_COLORS',  [
            'b4da35',
            '37af68',
            '64cf00',
            'cfcc00',
            'fdb735',
        ])

class ColorWidget(forms.Widget):

    def render(self, name, value, attrs=None):
        colors = COLORPICKER_COLORS
        return render_to_string("fhcore/templates/widgets/color_widget.html", locals())

class ColorField(models.CharField):
    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 10
        super(ColorField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        kwargs['widget'] = ColorWidget
        return super(ColorField, self).formfield(**kwargs)


'''
from PIL import Image
import os

try:
    from sorl.thumbnail.main import DjangoThumbnail
    def thumbnail(image_path, image_name):
        t = DjangoThumbnail(relative_source=image_name, requested_size=(80, 80))
        return u'<img style="float:left;padding-right:10px;" src="%s" alt="%s" />' % (t.absolute_url, image_name)
except ImportError:
    def thumbnail(image_path, image_name):
        return u'<img style="float:left;padding-right:10px;width:80px;height:80px" src="%s" alt="%s" />' % (image_path, image_name)

class AdminImageWidget(AdminFileWidget):
    """
      A FileField Widget that displays an image instead of a file path
     if the current file is an image.
      """
    def render(self, name, value, attrs=None):
        output = []
        file_name = str(value)
        if file_name:
            file_path = '%s%s' % (settings.MEDIA_URL, file_name)
            try:      # is image
                Image.open(os.path.join(settings.MEDIA_ROOT, file_name))
                output.append('<a target="_blank" href="%s">%s</a> <br />%s <a target="_blank" href="%s">%s</a> <br />%s ' % \
                              (file_path, thumbnail(file_path, file_name), _('Currently:'), file_path, file_name, _('Change:')))
            except IOError: # not image
                output.append('%s <a target="_blank" href="%s">%s</a> <br />%s ' % \
                              (_('Currently:'), file_path, file_name, _('Change:')))
                output.append(super(AdminFileWidget, self).render(name, value, attrs))
            return mark_safe(u''.join(output))
'''
